(* ACAP entries 

   Entries consist of a set attributes, as well as the default acl.

   Generally, entries are the thing that are passed around by the
   various routines.

   subdataset is going to be a pain.
*)

signature ENTRY =
sig
  structure Acl : ACL
  structure AcapTime : ACAPTIME

  type entry
  type metadata = {acl : Acl.acl option, attribute : string,
		   size : int list, value : Value.value}
  type storedata = {acl : Acl.acl option, attribute : string, 
		    value : Value.value option}
  type attribute = metadata

  (* raised when an attempt to store an illegal value is made to this
   dataset---returns attribute raised on *)
  exception EnforcedValidation of string

  (* raised when an attempt is made to store with a UNCHANGEDSINCE;
     returns the time it was last modified *)
  exception Modified of AcapTime.acaptime

  (* syntax error detected during store *)
  exception SyntaxError of string

  (* creates an entry, given it's complete name *)
  val create : string -> storedata list -> entry
  
  (* stores a list of attributes in this entry; this is where
     enforced validation takes place. returns NONE if this entry
     was deleted *)
  val store : entry -> storedata list -> AcapTime.acaptime option 
                    -> entry option

  (* get the entry's name *)
  val name : entry -> string

  (* get the entry's complete path *)
  val path : entry -> string

  (* gets the attributes *)
  val fetch : entry -> string list -> attribute list

  (* gets the matching attributes *)
  val search : entry -> (string -> bool) -> attribute list

  (* return the time of last modification *)
  val gettime : entry -> AcapTime.acaptime
end
