functor ThrottleFun(structure Queue : QUEUE) :> THROTTLE =
struct
  datatype hardresult = H_WAYTOOMANY | H_OK
  datatype softresult = S_TOOMANY of int | S_OK
  datatype result = OK

  datatype 'a throttle = 
    PassThrough of ('a -> unit) * (result -> unit)
  | SoftThrough of ('a -> unit) * (softresult -> unit) * 
    ('a Queue.queue * int * int * int) ref
  | HardThrough of ('a -> unit) * (hardresult -> unit) * 
    ('a Queue.queue * int) ref
  | SortThrough of ('a -> unit) * (result -> unit) * (('a * 'a) -> order) *
    'a Queue.queue ref

  (* create a hard throttle, plus what to call when done *)
  (* val createhard : int -> ('a -> unit) -> (unit -> unit) -> 'a throttle *)
  fun createhard lim f g =
    HardThrough(f, g, ref (Queue.empty, lim))

  (* create a soft throttle; the first number is the limit *)
  (* val createsoft : (int * int) -> ('a -> unit) -> (unit -> unit)
    -> 'a throttle *)
  fun createsoft (lim, pass) f g =
    SoftThrough(f, g, ref (Queue.empty, lim, pass, 0))

  (* create a sorting throttle *)
  (* val createsort : (('a * 'a) -> order) -> ('a -> unit) -> 
                   (result -> unit) -> 'a throttle *)
  fun createsort c f g = SortThrough (f, g, c, ref Queue.empty)

  (* a throttle that just passes through data *)
  (* val createpass : ('a -> unit) -> (unit -> unit) -> 'a throttle *)
  fun createpass f g = PassThrough(f, g)

  (* send data into the throttle *)
  (* val call : 'a throttle -> 'a -> unit *)
  fun call (HardThrough(f, g, cell as ref (q, lim))) a =
    if lim > 0 then cell := (Queue.add q a, lim - 1)
    else if lim = 0 then (cell := (Queue.empty, ~1); g H_WAYTOOMANY)
	 else ()
    | call (SoftThrough(f, g, cell as ref (q, lim, pass, tot))) a =
    if pass > 0 then (cell := (q, lim, pass - 1, tot + 1); f a)
    else if lim > 0 then cell := (Queue.add q a, lim - 1, pass, tot + 1)
	 else if lim = 0 then (cell := (Queue.empty, ~1, ~1, tot +1))
	      else (cell := (q, ~1, ~1, tot + 1))
    | call (PassThrough(f, g)) a = f a
    | call (SortThrough(f, g, c, cell as ref q)) a = cell := (Queue.add q a)


  fun quickSort comp [] = []
    | quickSort comp [x] = [x]
    | quickSort comp unsorted =
    let
      fun quick' [] l = l
	| quick' [x] l = x::l
	| quick' (a::bs) l = (* pivot by "a" *)
	let
	  fun partition (left, right, []) =
	    quick' left (a::quick' right l)
	    | partition (left, right, x::xs) =
	    if comp(x, a) <> GREATER then partition (x::left, right, xs)
	    else partition (left, x::right, xs)
	in
	  partition([], [], bs)
	end
    in
      quick' unsorted nil
    end


  (* indicates that no more data will be coming through the throttle;
     if the limit is exceeded it just throws out the data *)
  (* val done : 'a throttle -> unit *)
  fun done (HardThrough(f, g, ref (q, lim))) = 
    if lim >= 0 then (Queue.app q f; g H_OK)
    else ()
    | done (SoftThrough(f, g, ref (q, lim, pass, tot))) =
      if lim >= 0 then (Queue.app q f; g S_OK)
      else (g (S_TOOMANY tot))
    | done (PassThrough(f, g)) = g OK
    | done (SortThrough(f, g, c, ref q)) = 
	    (List.app f (quickSort c (Queue.toList q));
	     g OK)

end
