structure AcapTime :> ACAPTIME =
struct
  type acaptime = Time.time

  exception BadTimeString

  open Date

  val old = Time.zeroTime

  val future = Time.fromSeconds (valOf Int32.maxInt)

  val now = Time.now

  val compare = Time.compare

  fun intToString p = (StringCvt.padLeft #"0" p) o Int.toString

  fun monthToString Jan = "01"
    | monthToString Feb = "02"
    | monthToString Mar = "03"
    | monthToString Apr = "04"
    | monthToString May = "05"
    | monthToString Jun = "06"
    | monthToString Jul = "07"
    | monthToString Aug = "08"
    | monthToString Sep = "09"
    | monthToString Oct = "10"
    | monthToString Nov = "11"
    | monthToString Dec = "12"

  val iTS4 = intToString 4
  val iTS2 = intToString 2

  fun toString t = (* sigh *)
    let
      val d = Date.fromTimeUniv t
      val smallt = (Time.toMicroseconds (Time.-(t, Date.toTime d))
		    handle Time.Time => 0
			 | Overflow => 0)
    in
      (iTS4 (year d)) ^ 
      (monthToString (month d)) ^ 
      (iTS2 (day d)) ^
      (iTS2 (hour d)) ^
      (iTS2 (minute d)) ^
      (iTS2 (second d)) ^
      (Int32.toString smallt)
    end

  exception BadSplit
  exception BadMonth

  fun fromString s =
    let
      fun splitString n s = (String.extract (s, 0, SOME n),
			     String.extract (s, n, NONE))
      
      fun convMonth "01" = Jan
	| convMonth "02" = Feb
	| convMonth "03" = Mar
	| convMonth "04" = Apr
	| convMonth "05" = May
	| convMonth "06" = Jun
	| convMonth "07" = Jul
	| convMonth "08" = Aug
	| convMonth "09" = Sep
	| convMonth "10" = Oct
	| convMonth "11" = Nov
	| convMonth "12" = Dec
	| convMonth _ = raise BadMonth

      val sl = String.explode s

      val (year, s) = splitString 4 s
      val (month, s) = splitString 2 s
      val (day, s) = splitString 2 s
      val (hour, s) = splitString 2 s
      val (minute, s) = splitString 2 s
      val (second, smallt) = splitString 2 s

      val conv = (valOf o Int.fromString) 
      val year = conv year
      val month = convMonth month
      val day = conv day
      val hour = conv hour
      val minute = conv minute
      val second = conv second
      val smallt = (valOf o Int32.fromString) smallt

      val date = Date.date {year=year, month=month, day=day, hour=hour,
			    minute=minute, second=second, offset=NONE}

      val time = Time.+(Date.toTime date, Time.fromMicroseconds smallt)
    in
      time
    end handle _ => raise BadTimeString
    
  fun toValue t = Value.Single (toString t)
end
